package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.admin.assembler.ScoreAssembler;
import cn.com.qmth.stmms.ms.admin.dto.ScoreCheckDTO;
import cn.com.qmth.stmms.ms.admin.utils.WaterMarkUtils;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 成绩处理api
 * Created by zhengmin on 2017/1/6.
 */
@RestController
@RequestMapping("api/score")
public class ScoreApi {

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private ScoreAssembler scoreAssembler;

    /**
     * 查询
     *
     * @param workId     评卷工作id
     * @param examNumber 准考证号
     * @return
     */
    @RequestMapping(value = "search/byExamNumber", method = RequestMethod.GET)
    public ScoreCheckDTO getByExamNumber(@RequestParam Long workId,
                                         @RequestParam String examNumber) {
        Student student = studentRepo.findByWorkIdAndExamNumber(workId, examNumber);
        List<Paper> papers = paperRepo.findByWorkIdAndExamNumber(workId, examNumber);
        return scoreAssembler.toDTO(student, papers);
    }

    /**
     * 查询
     *
     * @param workId       评卷工作id
     * @param secretNumber 密号
     * @return
     */
    @RequestMapping(value = "search/bySecretNumber", method = RequestMethod.GET)
    public ScoreCheckDTO getBySecretNumber(@RequestParam Long workId,
                                           @RequestParam String secretNumber) {
        List<Paper> papers = paperRepo.findByWorkIdAndSecretNumber(workId, secretNumber);
        String examNumber = papers.get(0).getExamNumber();
        Student student = studentRepo.findByWorkIdAndExamNumber(workId, examNumber);

        return scoreAssembler.toDTO(student, papers);
    }

    @RequestMapping(value = "search/byTotalScore", method = RequestMethod.GET)
    public ResponseEntity getByTotalScore(@RequestParam final Long workId,
                                          @RequestParam final Long totalScore) {
//        List<Object[]> paperList = paperRepo.findByTotalScore(workId);
//        List<String> examNumbers = paperList.stream().filter(p -> {
//            Double t = (Double) p[1];
//            return t.longValue() > totalScore;
//        }).map(p -> {
//            return (String) p[0];
//        }).collect(Collectors.toList());
//
//        List<Paper> papers = paperRepo.findByWorkIdAndExamNumberIn(workId, examNumbers);
//        return papers.stream().map(paper -> {
//            String examNumber = paper.getExamNumber();
//            Student student = studentRepo.findByWorkIdAndExamNumber(workId, examNumber);
//            return scoreAssembler.toDTO(student, papers);
//        }).collect(Collectors.toList());


        List<ScoreCheckDTO> scoreDTOs = new ArrayList<>();
        List<Paper> papers = paperRepo.findByWorkId(workId);
        Map<String, List<Paper>> group = papers.stream().collect(Collectors.groupingBy(Paper::getExamNumber));
        group.forEach((k, v) -> {
            //long ct = v.stream().mapToDouble(Paper::getScore).filter(s -> s >= hs).count();
            long studentTotalScore = 0;
            for (Paper p : v) {
                studentTotalScore += p.getScore();
            }
            if (v.size() == 3 && studentTotalScore >= totalScore) {

                Student student = studentRepo.findByWorkIdAndExamNumber(workId, k);
                scoreDTOs.add(scoreAssembler.toDTO(student, v));
            }
        });
        return ResponseEntity.ok(scoreDTOs);
    }

    /**
     * 查询任务两科成绩高于hs，另外一颗低于ls
     *
     * @param workId 评卷工作id
     * @param hs     高分
     * @param ls     低分
     * @return
     */
    @RequestMapping(value = "search/byRange", method = RequestMethod.GET)
    public ResponseEntity getByScoreRange(@RequestParam Long workId,
                                          @RequestParam Double hs,
                                          @RequestParam Double ls) {
        List<ScoreCheckDTO> scoreDTOs = new ArrayList<>();
        List<Paper> papers = paperRepo.findByScoreRange(hs, ls);
        Map<String, List<Paper>> group = papers.stream().collect(Collectors.groupingBy(Paper::getExamNumber));
        group.forEach((k, v) -> {
            long ct = v.stream().mapToDouble(Paper::getScore).filter(s -> s >= hs).count();
            if (v.size() == 3 && ct == 2) {
                Student student = studentRepo.findByWorkIdAndExamNumber(workId, k);
                scoreDTOs.add(scoreAssembler.toDTO(student, v));
            }
        });
        return ResponseEntity.ok(scoreDTOs);
    }

    /**
     * 更新分数
     *
     * @param paperId 试卷id
     * @param body    分数
     * @return
     */
    @RequestMapping(value = "update/{paperId}", method = RequestMethod.PUT)
    public ResponseEntity updateScore(@PathVariable Long paperId, @RequestBody ModelMap body) {
        Paper paper = paperRepo.findOne(paperId);
        if (paper == null || body.get("score") == null) {
            return new ResponseEntity(HttpStatus.OK);
        }
        Double score = Double.valueOf(body.get("score").toString());
        if (score >= 0 && score <= 100) {
            paper.setScore(score);
            paperRepo.save(paper);
        }
        return new ResponseEntity(HttpStatus.OK);
    }

    @RequestMapping(value = "merge/{subject}", method = RequestMethod.GET)
    public ResponseEntity merge(@PathVariable Subject subject) {
        Work activedWork = workRepo.findByActiveTrue();
        List<Paper> papers = paperRepo.findByWorkIdAndSubject(activedWork.getId(), subject);
        Iterator<Paper> paperIterator = papers.iterator();
        long count = 0;
        while (paperIterator.hasNext()) {
            Paper paper = paperIterator.next();
            List<MarkTask> markTasks = markTaskRepo.findByPaperIdAndStage(paper.getId(), MarkStage.SCORE);
            long leftCount = markTasks.stream().filter(i -> i.getResult() == null).count();
            if (leftCount == 0) {
                OptionalDouble finalScore = markTasks.stream().map(m -> m.getResult())
                        .mapToInt(Integer::valueOf).average();
                double fs = Math.round(finalScore.orElse(0));
                paper.setScore(fs);
                paperRepo.save(paper);
                count++;
            }
            if (count % 500 == 0) {
                System.out.println(subject.toString() + ":" + count);
            }
        }
        return ResponseEntity.ok(null);
    }

    @Value("${sys.config.imageDir}")
    private String imagePath;

    @Value("${sys.config.watermark}")
    private String watermark;

    private static int watermarkTotal;
    private static int watermarkFinished;
    private static boolean watermarking = false;

    @PostMapping("/watermark")
    public void watermark(@RequestParam("workId") Long workId) {

        if (!watermarking) {
            watermarking = true;
            watermarkTotal = 0;
            watermarkFinished = 0;

            final Font font = new Font("宋体", Font.PLAIN, 60);                     //水印字体
            final Color color = new Color(254, 3, 10, 255);                               //水印图片色彩以及透明度
            final List<Paper> papers = paperRepo.findByWorkId(workId);
            watermarkTotal = papers.size();

            new Thread(() -> {

                for (int i = 0; i < watermarkTotal; i++) {
                    watermarkFinished = i + 1;
                    Paper paper = papers.get(i);
                    Double score = paper.getInspectScore() == null ? paper.getScore() : paper.getInspectScore();
                    String examNumber = paper.getExamNumber();
                    String areaCode = paper.getAreaCode();
                    Subject subject = paper.getSubject();

                    String imageFile = imagePath + "/" + workId + "/" + subject.toString() + "/" + areaCode + "/" + examNumber + ".jpg";
                    String watermarkFile = watermark + "/" + workId + "/" + subject.toString() + "/" + areaCode + "/" + examNumber + ".jpg";

                    File targetFile = new File(watermarkFile);
                    targetFile.getParentFile().mkdirs();//创建父级目录
                    String txt = "【" + paper.getExamNumber() + "】【" + paper.getStudentName() + "】【" + String.valueOf(score.intValue()) + "分】";
                    String text = new String(txt.getBytes(), Charset.forName("UTF-8"));
//                    WaterMarkUtils.addWaterMark(imageFile, watermarkFile, text, color, font);
                    try {
                        WaterMarkUtils.addTextWatermark(text, "jpg", imageFile, watermarkFile);
                        System.out.println(watermarkFile + "生成成功");
                    } catch (IOException e) {
                        System.out.println(watermarkFile + "生成失败");
                    }

                }

                watermarking = false;
            }).start();
        } else {
            throw new RuntimeException("请勿重复生成");
        }


    }

    @GetMapping("/watermark")
    public Map<String, Object> watermarkProgress(@RequestParam("workId") Long workId) {
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("watermarkTotal", watermarkTotal);
        resultMap.put("watermarkFinished", watermarkFinished);
        resultMap.put("watermarking", watermarking);
        return resultMap;
    }
}
